@@ -1,7 +1,3 @@
-use std::borrow::Cow;
-
-use crate::parsers::message::MessageStream;
-
 /*
  * Copyright Stalwart Labs, Minter Ltd. See the COPYING
  * file at the top-level directory of this distribution.
@@ -13,6 +9,8 @@ use crate::parsers::message::MessageStream;
  * except according to those terms.
  */
 
+use crate::parsers::message::MessageStream;
+
 pub mod base64;
 pub mod charsets;
 pub mod encoded_word;
@@ -20,5 +18,10 @@ pub mod hex;
 pub mod html;
 pub mod quoted_printable;
 
-pub type DecodeFnc<'x> =
-    fn(&MessageStream<'x>, usize, &[u8], bool) -> (usize, Option<Cow<'x, [u8]>>);
+pub type DecodeFnc<'x> = fn(&MessageStream<'x>, usize, &[u8], bool) -> (usize, DecodeResult);
+
+pub enum DecodeResult {
+    Owned(Vec<u8>),
+    Borrowed((usize, usize)),
+    Empty,
+}
